@@ -567,7 +567,7 @@ public class Haval extends BaseHash
   {
     if (valid == null)
       {
-        valid = new Boolean(DIGEST0.equals(Util.toString(new Haval().digest())));
+        valid = Boolean.valueOf(DIGEST0.equals(Util.toString(new Haval().digest())));
       }
     return valid.booleanValue();
   }
