@@ -106,6 +106,10 @@ public class JSpinner extends JComponent
       add(ftf);
       ftf.setValue(spinner.getValue());
       ftf.addPropertyChangeListener(this);
+      if(getComponentOrientation().isLeftToRight())
+	ftf.setHorizontalAlignment(JTextField.RIGHT);
+      else
+	ftf.setHorizontalAlignment(JTextField.LEFT);
       spinner.addChangeListener(this);
     }
 
