@@ -1,5 +1,5 @@
 /* CompositeView.java -- An abstract view that manages child views
-   Copyright (C) 2005  Free Software Foundation, Inc.
+   Copyright (C) 2005, 2006  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -218,21 +218,24 @@ public abstract class CompositeView
     throws BadLocationException
   {
     int childIndex = getViewIndex(pos, bias);
+    if (childIndex == -1)
+      throw new BadLocationException("Position " + pos + " is not represented by view.", pos);
+      
     Shape ret = null;
-    if (childIndex != -1)
-      {
-        View child = getView(childIndex);
-        Shape childAlloc = getChildAllocation(childIndex, a);
-        if (childAlloc == null)
-          ret = createDefaultLocation(a, bias);
-        Shape result = child.modelToView(pos, childAlloc, bias);
-        if (result != null)
-          ret = result;
-        else
-          ret =  createDefaultLocation(a, bias);
-      }
-    else
+
+    View child = getView(childIndex);
+    Shape childAlloc = getChildAllocation(childIndex, a);
+    
+    if (childAlloc == null)
       ret = createDefaultLocation(a, bias);
+    
+    Shape result = child.modelToView(pos, childAlloc, bias);
+
+    if (result != null)
+      ret = result;
+    else
+      ret =  createDefaultLocation(a, bias);
+
     return ret;
   }
 
@@ -301,7 +304,7 @@ public abstract class CompositeView
       {
         Rectangle r = getInsideAllocation(a);
         View view = getViewAtPoint((int) x, (int) y, r);
-        return view.viewToModel(x, y, a, b);
+        return view.viewToModel(x, y, r, b);
       }
     return 0;
   }
@@ -630,8 +633,51 @@ public abstract class CompositeView
                                                     Position.Bias[] biasRet)
     throws BadLocationException
   {
-    // FIXME: Implement this correctly.
-    return pos;
+    // TODO: It is unknown to me how this method has to be implemented and
+    // there is no specification telling me how to do it properly. Therefore
+    // the implementation was done for cases that are known.
+    //
+    // If this method ever happens to act silly for your particular case then
+    // it is likely that it is a cause of not knowing about your case when it
+    // was implemented first. You are free to fix the behavior.
+    //
+    // Here are the assumptions that lead to the implementation:
+    // If direction is NORTH chose the View preceding the one that contains the
+    // offset 'pos' (imagine the views are stacked on top of each other where
+    // the top is 0 and the bottom is getViewCount()-1.
+    // Consecutively when the direction is SOUTH the View following the one
+    // the offset 'pos' lies in is questioned.
+    //
+    // This limitation is described as PR 27345.
+    int index = getViewIndex(pos, b);
+    View v = null;
+    
+    if (index == -1)
+      return pos;
+
+    switch (direction)
+    {
+      case NORTH:
+        // If we cannot calculate a proper offset return the one that was
+        // provided.
+        if (index <= 0)
+          return pos;
+        
+        v = getView(index - 1);
+        break;
+      case SOUTH:
+        // If we cannot calculate a proper offset return the one that was
+        // provided.
+        if (index >= getViewCount() - 1)
+          return pos;
+        
+        v = getView(index + 1);
+        break;
+      default:
+          throw new IllegalArgumentException();
+    }
+    
+    return v.getNextVisualPositionFrom(pos, b, a, direction, biasRet);
   }
 
   /**
@@ -664,8 +710,55 @@ public abstract class CompositeView
                                                   Position.Bias[] biasRet)
     throws BadLocationException
   {
-    // FIXME: Implement this correctly.
-    return pos;
+    // TODO: It is unknown to me how this method has to be implemented and
+    // there is no specification telling me how to do it properly. Therefore
+    // the implementation was done for cases that are known.
+    //
+    // If this method ever happens to act silly for your particular case then
+    // it is likely that it is a cause of not knowing about your case when it
+    // was implemented first. You are free to fix the behavior.
+    //
+    // Here are the assumptions that lead to the implementation:
+    // If direction is EAST increase the offset by one and ask the View to
+    // which that index belong to calculate the 'next visual position'.
+    // If the direction is WEST do the same with offset 'pos' being decreased
+    // by one.
+    // This behavior will fail in a right-to-left or bidi environment!
+    //
+    // This limitation is described as PR 27346.
+    int index;
+    
+    View v = null;
+    
+    switch (direction)
+    {
+      case EAST:
+        index = getViewIndex(pos + 1, b);
+        // If we cannot calculate a proper offset return the one that was
+        // provided.
+        if (index == -1)
+          return pos;
+        
+        v  = getView(index);
+        break;
+      case WEST:
+        index = getViewIndex(pos - 1, b);
+        // If we cannot calculate a proper offset return the one that was
+        // provided.
+        if (index == -1)
+          return pos;
+        
+        v  = getView(index);
+        break;
+      default:
+        throw new IllegalArgumentException();
+    }
+    
+    return v.getNextVisualPositionFrom(pos,
+                                       b,
+                                       a,
+                                       direction,
+                                       biasRet);
   }
 
   /**
