@@ -1,5 +1,5 @@
 /* GapContent.java --
-   Copyright (C) 2002, 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004, 2005, 2006 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -39,8 +39,10 @@ exception statement from your version. */
 package javax.swing.text;
 
 import java.io.Serializable;
+import java.lang.ref.WeakReference;
 import java.util.ArrayList;
 import java.util.Collections;
+import java.util.Comparator;
 import java.util.Iterator;
 import java.util.ListIterator;
 import java.util.Vector;
@@ -68,8 +70,8 @@ public class GapContent
   /**
    * A {@link Position} implementation for <code>GapContent</code>.
    */
-  class GapContentPosition
-      implements Position, Comparable
+  private class GapContentPosition
+    implements Position, Comparable
   {
 
     /** The index within the buffer array. */
@@ -123,15 +125,14 @@ public class GapContent
       assert mark <= gapStart || mark >= gapEnd : "mark: " + mark
                                                + ", gapStart: " + gapStart
                                                + ", gapEnd: " + gapEnd;
-
-      if (mark <= gapEnd)
+      if (mark <= gapStart)
         return mark;
       else
         return mark - (gapEnd - gapStart);
     }
   }
 
-  class InsertUndo extends AbstractUndoableEdit
+  private class InsertUndo extends AbstractUndoableEdit
   {
     public int where, length;
     String text;
@@ -170,7 +171,7 @@ public class GapContent
     
   }
   
-  class UndoRemove extends AbstractUndoableEdit
+  private class UndoRemove extends AbstractUndoableEdit
   {
     public int where;
     String text;
@@ -207,7 +208,41 @@ public class GapContent
     }
     
   }
-  
+
+  /**
+   * Compares WeakReference objects in a List by comparing the referenced
+   * objects instead.
+   *
+   * @author Roman Kennke (kennke@aicas.com)
+   */
+  private class WeakPositionComparator
+    implements Comparator
+  {
+
+    /**
+     * Compares two objects of type WeakReference. The objects are compared
+     * using the referenced objects compareTo() method.
+     */
+    public int compare(Object o1, Object o2)
+    {
+      // Unwrap references.
+      if (o1 instanceof WeakReference)
+        o1 = ((WeakReference) o1).get();
+      if (o2 instanceof WeakReference)
+        o2 = ((WeakReference) o2).get();
+
+      GapContentPosition p1 = (GapContentPosition) o1;
+      GapContentPosition p2 = (GapContentPosition) o2;
+
+      int retVal;
+      if (p1 == null || p2 == null)
+        retVal = -1;
+      else
+        retVal = p1.compareTo(p2);
+      return retVal;
+    }
+  }
+
   /** The serialization UID (compatible with JDK1.5). */
   private static final long serialVersionUID = -6226052713477823730L;
 
@@ -234,9 +269,10 @@ public class GapContent
 
   /**
    * The positions generated by this GapContent. They are kept in an ordered
-   * fashion, so they can be looked up easily.
+   * fashion, so they can be looked up easily. The value objects will be
+   * WeakReference objects that in turn hold GapContentPosition objects.
    */
-  ArrayList positions;
+  private ArrayList positions;
 
   /**
    * Creates a new GapContent object.
@@ -253,6 +289,7 @@ public class GapContent
    */
   public GapContent(int size)
   {
+    size = Math.max(size, 2);
     buffer = (char[]) allocateArray(size);
     gapStart = 1;
     gapEnd = size;
@@ -311,8 +348,12 @@ public class GapContent
     int length = length();
     int strLen = str.length();
 
-    if (where >= length)
-      throw new BadLocationException("the where argument cannot be greater"
+    if (where < 0)
+      throw new BadLocationException("The where argument cannot be smaller"
+                                     + " than the zero", where);
+
+    if (where > length)
+      throw new BadLocationException("The where argument cannot be greater"
           + " than the content length", where);
 
     replace(where, 0, str.toCharArray(), strLen);
@@ -335,14 +376,11 @@ public class GapContent
   {
     // check arguments
     int length = length();
-
-    if (where >= length)
-      throw new BadLocationException("the where argument cannot be greater"
-          + " than the content length", where);
-    if ((where + nitems) > length)
+    
+    if ((where + nitems) >= length)
       throw new BadLocationException("where + nitems cannot be greater"
           + " than the content length", where + nitems);
-
+    
     String removedText = getString(where, nitems);
     replace(where, nitems, null, 0);
 
@@ -399,6 +437,8 @@ public class GapContent
   {
     // check arguments
     int length = length();
+    if (where < 0)
+      throw new BadLocationException("the where argument may not be below zero", where);
     if (where >= length)
       throw new BadLocationException("the where argument cannot be greater"
           + " than the content length", where);
@@ -447,18 +487,22 @@ public class GapContent
       throw new BadLocationException("The offset was out of the bounds of this"
           + " buffer", offset);
 
+    clearPositionReferences();
+
     // We store the actual array index in the GapContentPosition. The real
     // offset is then calculated in the GapContentPosition.
     int mark = offset;
     if (offset >= gapStart)
       mark += gapEnd - gapStart;
     GapContentPosition pos = new GapContentPosition(mark);
+    WeakReference r = new WeakReference(pos);
 
     // Add this into our list in a sorted fashion.
-    int index = Collections.binarySearch(positions, pos);
+    int index = Collections.binarySearch(positions, r,
+                                         new WeakPositionComparator());
     if (index < 0)
       index = -(index + 1);
-    positions.add(index, pos);
+    positions.add(index, r);
     return pos;
   }
 
@@ -558,7 +602,7 @@ public class GapContent
 
     assert newGapEnd > gapEnd : "The new gap end must be greater than the "
                                 + "old gap end.";
-    setPositionsInRange(gapEnd, newGapEnd - gapEnd, newGapEnd + 1);
+    setPositionsInRange(gapEnd, newGapEnd - gapEnd, newGapEnd);
     gapEnd = newGapEnd;
   }
 
@@ -567,7 +611,7 @@ public class GapContent
    * 
    * @return the allocated buffer array
    */
-  protected Object getArray()
+  protected final Object getArray()
   {
     return buffer;
   }
@@ -598,6 +642,10 @@ public class GapContent
     if (addItems != null)
       {
         System.arraycopy(addItems, 0, buffer, gapStart, addSize);
+        
+        
+        resetMarksAtZero();
+        
         gapStart += addSize;
       }
   }
@@ -643,19 +691,30 @@ public class GapContent
     int endOffset = offset + length;
 
     int index1 = Collections.binarySearch(positions,
-                                          new GapContentPosition(offset));
+                                          new GapContentPosition(offset),
+                                          new WeakPositionComparator());
     if (index1 < 0)
       index1 = -(index1 + 1);
 
     // Search the first index with the specified offset. The binarySearch does
     // not necessarily find the first one.
-    while (index1 > 0
-        && ((GapContentPosition) positions.get(index1 - 1)).mark == offset)
-      index1--;
+    while (index1 > 0)
+      {
+        WeakReference r = (WeakReference) positions.get(index1 - 1);
+        GapContentPosition p = (GapContentPosition) r.get();
+        if (p != null && p.mark == offset || p == null)
+          index1--;
+        else
+          break;
+      }
 
     for (ListIterator i = positions.listIterator(index1); i.hasNext();)
       {
-        GapContentPosition p = (GapContentPosition) i.next();
+        WeakReference r = (WeakReference) i.next();
+        GapContentPosition p = (GapContentPosition) r.get();
+        if (p == null)
+          continue;
+
         if (p.mark > endOffset)
           break;
         if (p.mark >= offset && p.mark <= endOffset)
@@ -673,24 +732,35 @@ public class GapContent
    * @param length the length of the range to search
    * @param value the new value for each mark
    */
-  void setPositionsInRange(int offset, int length, int value)
+  private void setPositionsInRange(int offset, int length, int value)
   {
     int endOffset = offset + length;
 
     int index1 = Collections.binarySearch(positions,
-                                          new GapContentPosition(offset));
+                                          new GapContentPosition(offset),
+                                          new WeakPositionComparator());
     if (index1 < 0)
       index1 = -(index1 + 1);
 
     // Search the first index with the specified offset. The binarySearch does
     // not necessarily find the first one.
-    while (index1 > 0
-        && ((GapContentPosition) positions.get(index1 - 1)).mark == offset)
-      index1--;
+    while (index1 > 0)
+      {
+        WeakReference r = (WeakReference) positions.get(index1 - 1);
+        GapContentPosition p = (GapContentPosition) r.get();
+        if (p != null && p.mark == offset || p == null)
+          index1--;
+        else
+          break;
+      }
 
     for (ListIterator i = positions.listIterator(index1); i.hasNext();)
       {
-        GapContentPosition p = (GapContentPosition) i.next();
+        WeakReference r = (WeakReference) i.next();
+        GapContentPosition p = (GapContentPosition) r.get();
+        if (p == null)
+          continue;
+
         if (p.mark > endOffset)
           break;
         
@@ -708,23 +778,35 @@ public class GapContent
    * @param length the length of the range to search
    * @param incr the increment
    */
-  void adjustPositionsInRange(int offset, int length, int incr)
+  private void adjustPositionsInRange(int offset, int length, int incr)
   {
     int endOffset = offset + length;
 
     int index1 = Collections.binarySearch(positions,
-                                          new GapContentPosition(offset));
+                                          new GapContentPosition(offset),
+                                          new WeakPositionComparator());
     if (index1 < 0)
       index1 = -(index1 + 1);
 
     // Search the first index with the specified offset. The binarySearch does
     // not necessarily find the first one.
-    while (index1 > 0
-        && ((GapContentPosition) positions.get(index1 - 1)).mark == offset)
-      index1--;
+    while (index1 > 0)
+      {
+        WeakReference r = (WeakReference) positions.get(index1 - 1);
+        GapContentPosition p = (GapContentPosition) r.get();
+        if (p != null && p.mark == offset || p == null)
+          index1--;
+        else
+          break;
+      }
+
     for (ListIterator i = positions.listIterator(index1); i.hasNext();)
       {
-        GapContentPosition p = (GapContentPosition) i.next();
+        WeakReference r = (WeakReference) i.next();
+        GapContentPosition p = (GapContentPosition) r.get();
+        if (p == null)
+          continue;
+
         if (p.mark > endOffset)
           break;
 
@@ -747,6 +829,17 @@ public class GapContent
     setPositionsInRange(gapEnd, 0, 0);
   }
 
+  /**
+   * @specnote This method is not very well specified and the positions vector
+   *           is implementation specific. The undo positions are managed
+   *           differently in this implementation, this method is only here
+   *           for binary compatibility.
+   */
+  protected void updateUndoPositions(Vector positions, int offset, int length)
+  {
+    // We do nothing here.
+  }
+
   /**
    * Outputs debugging info to System.err. It prints out the buffer array,
    * the gapStart is marked by a &lt; sign, the gapEnd is marked by a &gt;
@@ -777,8 +870,23 @@ public class GapContent
   {
     for (Iterator i = positions.iterator(); i.hasNext();)
       {
-        GapContentPosition pos = (GapContentPosition) i.next();
+        WeakReference r = (WeakReference) i.next();
+        GapContentPosition pos = (GapContentPosition) r.get();
         System.err.println("position at: " + pos.mark);
       }
   }
+
+  /**
+   * Clears all GC'ed references in the positions array.
+   */
+  private void clearPositionReferences()
+  {
+    Iterator i = positions.iterator();
+    while (i.hasNext())
+      {
+        WeakReference r = (WeakReference) i.next();
+        if (r.get() == null)
+          i.remove();
+      }
+  }
 }
